#!/usr/bin/perl
#
# hobbs2.lisp.pl
#
# George Ferguson, ferguson@cs.rochester.edu, 30 Aug 2002
# Time-stamp: <Thu Sep  5 16:31:58 EDT 2002 ferguson>
#
# Extract marked axioms from text, fed them to the CUP parser, and
# write out the results in a useful form. The parser can only handle
# one formula in its input at a time, so we just rerun it for each
# axiom. This could be improved (in fact, we could do the axiom extraction
# in the lexical analyzer and add to the grammar to allow multiple formulas
# in an input stream), but who cares?
#
# Extended by gf to use a snippet of Lisp code to add an explicit forall
# for the implicit universally quantified variables in Jerry's notation.
# This has teh nice side-effect that the forms get pretty-printed by Lisp.
#

$PARSER = "java -classpath .:java_cup-0.10k parser";

$POSTPROCESSOR = "mlisp -L explicit -e '(lisp2explicit)' -e '(exit 0 :quiet t)'";

open(POST, "|$POSTPROCESSOR") || die("couldn't run $POSTPROCESSOR: $!\n");

while (<>) {
    if (/^<axiom id="([^"]+)">/) {
	$id = $1;
	$gather = 1;
    } elsif (/^<\/axiom>/) {
	$text =~ s/\n/ /g;
	run_parser();
	$text = '';
	$gather = 0;
    } elsif ($gather) {
	$text .= $_;
    }
}

close(POST);
exit(0);

sub run_parser {
    open(PIPE, "echo '$text' | $PARSER|") || die("couldn't run parser: $!\n");
    undef $/;
    $form = <PIPE>;
    $/ = "\n";
    close(PIPE);
    chomp($form);
    print POST "(defaxiom \"$id\"\n  $form)\n\n";
}

